@@ -48,6 +48,7 @@ typedef struct {
 	EmpathyContactFactory   *contact_factory;
 	McAccount               *account;
 	EmpathyTpContactFactory *tp_contact_factory;
+	GtkFileChooser          *chooser_dialog;
 
 	gulong ready_handler_id;
 
@@ -213,7 +214,7 @@ empathy_avatar_chooser_init (EmpathyAvatarChooser *chooser)
 			  G_CALLBACK (avatar_chooser_clicked_cb),
 			  chooser);
 
-	priv->contact_factory = empathy_contact_factory_new ();
+	priv->contact_factory = empathy_contact_factory_dup_singleton ();
 
 	empathy_avatar_chooser_set (chooser, NULL);
 }
@@ -577,13 +578,11 @@ avatar_chooser_clear_image (EmpathyAvatarChooser *chooser)
 	EmpathyAvatarChooserPriv *priv = GET_PRIV (chooser);
 	GtkWidget *image;
 
-	if (priv->avatar == NULL) {
-		return;
+	if (priv->avatar != NULL) {
+		empathy_avatar_unref (priv->avatar);
+		priv->avatar = NULL;
 	}
 
-	empathy_avatar_unref (priv->avatar);
-	priv->avatar = NULL;
-
 	image = gtk_image_new_from_icon_name ("stock_person", GTK_ICON_SIZE_DIALOG);
 	gtk_button_set_image (GTK_BUTTON (chooser), image);
 	g_signal_emit (chooser, signals[CHANGED], 0);
@@ -897,6 +896,8 @@ avatar_chooser_response_cb (GtkWidget            *widget,
 {
 	EmpathyAvatarChooserPriv *priv = GET_PRIV (chooser);
 
+	priv->chooser_dialog = NULL;
+
 	if (response == GTK_RESPONSE_CANCEL) {
 		goto out;
 	}
@@ -942,8 +943,14 @@ avatar_chooser_clicked_cb (GtkWidget            *button,
 	const gchar    *default_dir = DEFAULT_DIR;
 	const gchar    *pics_dir;
 	GtkFileFilter  *filter;
+	EmpathyAvatarChooserPriv *priv = GET_PRIV (chooser);
 
-	chooser_dialog = GTK_FILE_CHOOSER (
+	if (priv->chooser_dialog) {
+		gtk_window_present (GTK_WINDOW (priv->chooser_dialog));
+		return;
+	}
+
+	priv->chooser_dialog = GTK_FILE_CHOOSER (
 		gtk_file_chooser_dialog_new (_("Select Your Avatar Image"),
 					     empathy_get_toplevel_window (GTK_WIDGET (chooser)),
 					     GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -954,6 +961,8 @@ avatar_chooser_clicked_cb (GtkWidget            *button,
 					     GTK_STOCK_OPEN,
 					     GTK_RESPONSE_OK,
 					     NULL));
+	chooser_dialog = priv->chooser_dialog;
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (chooser_dialog), TRUE);
 
 	/* Get special dirs */
 	empathy_conf_get_string (empathy_conf_get (),
