@@ -36,6 +36,7 @@
 #include <gio/gunixoutputstream.h>
 
 #include <telepathy-glib/proxy-subclass.h>
+#include <telepathy-glib/util.h>
 
 #include "empathy-tp-file.h"
 #include "empathy-contact-factory.h"
@@ -43,6 +44,8 @@
 #include "empathy-time.h"
 #include "empathy-utils.h"
 
+#include "extensions/extensions.h"
+
 #define DEBUG_FLAG EMPATHY_DEBUG_FT
 #include "empathy-debug.h"
 
@@ -283,19 +286,19 @@ struct _EmpathyTpFilePriv {
   GOutputStream *out_stream;
 
   /* org.freedesktop.Telepathy.Channel.Type.FileTransfer D-Bus properties */
-  EmpFileTransferState state;
+  TpFileTransferState state;
   gchar *content_type;
   gchar *filename;
   guint64 size;
-  EmpFileHashType content_hash_type;
+  TpFileHashType content_hash_type;
   gchar *content_hash;
   gchar *description;
   guint64 transferred_bytes;
 
   gboolean incoming;
-  EmpFileTransferStateChangeReason state_change_reason;
+  TpFileTransferStateChangeReason state_change_reason;
   time_t start_time;
-  gchar *unix_socket_path;
+  GValue *socket_address;
   GCancellable *cancellable;
 };
 
@@ -334,13 +337,13 @@ tp_file_invalidated_cb (TpProxy       *proxy,
 {
   DEBUG ("Channel invalidated: %s", message);
 
-  if (tp_file->priv->state != EMP_FILE_TRANSFER_STATE_COMPLETED &&
-      tp_file->priv->state != EMP_FILE_TRANSFER_STATE_CANCELLED)
+  if (tp_file->priv->state != TP_FILE_TRANSFER_STATE_COMPLETED &&
+      tp_file->priv->state != TP_FILE_TRANSFER_STATE_CANCELLED)
     {
       /* The channel is not in a finished state, an error occured */
-      tp_file->priv->state = EMP_FILE_TRANSFER_STATE_CANCELLED;
+      tp_file->priv->state = TP_FILE_TRANSFER_STATE_CANCELLED;
       tp_file->priv->state_change_reason =
-          EMP_FILE_TRANSFER_STATE_CHANGE_REASON_LOCAL_ERROR;
+          TP_FILE_TRANSFER_STATE_CHANGE_REASON_LOCAL_ERROR;
       g_object_notify (G_OBJECT (tp_file), "state");
     }
 }
@@ -352,12 +355,10 @@ tp_file_finalize (GObject *object)
 
   if (tp_file->priv->channel)
     {
-      DEBUG ("Closing channel..");
       g_signal_handlers_disconnect_by_func (tp_file->priv->channel,
           tp_file_invalidated_cb, object);
-      tp_cli_channel_call_close (tp_file->priv->channel, -1, NULL, NULL,
-          NULL, NULL);
       g_object_unref (tp_file->priv->channel);
+      tp_file->priv->channel = NULL;
     }
 
   if (tp_file->priv->factory)
@@ -370,7 +371,8 @@ tp_file_finalize (GObject *object)
     }
 
   g_free (tp_file->priv->filename);
-  g_free (tp_file->priv->unix_socket_path);
+  if (tp_file->priv->socket_address != NULL)
+    tp_g_value_slice_free (tp_file->priv->socket_address);
   g_free (tp_file->priv->description);
   g_free (tp_file->priv->content_hash);
   g_free (tp_file->priv->content_type);
@@ -394,8 +396,8 @@ static void
 tp_file_start_transfer (EmpathyTpFile *tp_file)
 {
   gint fd;
-  size_t path_len;
   struct sockaddr_un addr;
+  GArray *array;
 
   fd = socket (PF_UNIX, SOCK_STREAM, 0);
   if (fd < 0)
@@ -405,10 +407,11 @@ tp_file_start_transfer (EmpathyTpFile *tp_file)
       return;
     }
 
+  array = g_value_get_boxed (tp_file->priv->socket_address);
+
   memset (&addr, 0, sizeof (addr));
   addr.sun_family = AF_UNIX;
-  path_len = strlen (tp_file->priv->unix_socket_path);
-  strncpy (addr.sun_path, tp_file->priv->unix_socket_path, path_len);
+  strncpy (addr.sun_path, array->data, array->len);
 
   if (connect (fd, (struct sockaddr*) &addr, sizeof (addr)) < 0)
     {
@@ -443,7 +446,7 @@ tp_file_start_transfer (EmpathyTpFile *tp_file)
 }
 
 static void
-tp_file_state_changed_cb (TpProxy *proxy,
+tp_file_state_changed_cb (TpChannel *channel,
                           guint state,
                           guint reason,
                           gpointer user_data,
@@ -465,8 +468,8 @@ tp_file_state_changed_cb (TpProxy *proxy,
   /* If the channel is open AND we have the socket path, we can start the
    * transfer. The socket path could be NULL if we are not doing the actual
    * data transfer but are just an observer for the channel. */
-  if (state == EMP_FILE_TRANSFER_STATE_OPEN &&
-      tp_file->priv->unix_socket_path != NULL)
+  if (state == TP_FILE_TRANSFER_STATE_OPEN &&
+      tp_file->priv->socket_address != NULL)
     tp_file_start_transfer (tp_file);
 
   tp_file->priv->state = state;
@@ -476,7 +479,7 @@ tp_file_state_changed_cb (TpProxy *proxy,
 }
 
 static void
-tp_file_transferred_bytes_changed_cb (TpProxy *proxy,
+tp_file_transferred_bytes_changed_cb (TpChannel *channel,
                                       guint64 count,
                                       gpointer user_data,
                                       GObject *weak_object)
@@ -507,18 +510,18 @@ tp_file_constructor (GType type,
 
   tp_file = EMPATHY_TP_FILE (file_obj);
 
-  tp_file->priv->factory = empathy_contact_factory_new ();
-  tp_file->priv->mc = empathy_mission_control_new ();
+  tp_file->priv->factory = empathy_contact_factory_dup_singleton ();
+  tp_file->priv->mc = empathy_mission_control_dup_singleton ();
 
   g_signal_connect (tp_file->priv->channel, "invalidated",
     G_CALLBACK (tp_file_invalidated_cb), tp_file);
 
-  emp_cli_channel_type_file_transfer_connect_to_file_transfer_state_changed (
-      TP_PROXY (tp_file->priv->channel), tp_file_state_changed_cb, NULL, NULL,
+  tp_cli_channel_type_file_transfer_connect_to_file_transfer_state_changed (
+      tp_file->priv->channel, tp_file_state_changed_cb, NULL, NULL,
       G_OBJECT (tp_file), NULL);
 
-  emp_cli_channel_type_file_transfer_connect_to_transferred_bytes_changed (
-      TP_PROXY (tp_file->priv->channel), tp_file_transferred_bytes_changed_cb,
+  tp_cli_channel_type_file_transfer_connect_to_transferred_bytes_changed (
+      tp_file->priv->channel, tp_file_transferred_bytes_changed_cb,
       NULL, NULL, G_OBJECT (tp_file), NULL);
 
   account = empathy_channel_get_account (tp_file->priv->channel);
@@ -528,7 +531,7 @@ tp_file_constructor (GType type,
       tp_file->priv->factory, account, (guint) handle);
 
   tp_cli_dbus_properties_run_get_all (tp_file->priv->channel,
-      -1, EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, &properties, NULL, NULL);
+      -1, TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, &properties, NULL, NULL);
 
   tp_cli_dbus_properties_run_get (tp_file->priv->channel,
       -1, TP_IFACE_CHANNEL, "Requested", &requested, NULL, NULL);
@@ -540,7 +543,7 @@ tp_file_constructor (GType type,
       g_hash_table_lookup (properties, "State"));
 
   tp_file->priv->state_change_reason =
-      EMP_FILE_TRANSFER_STATE_CHANGE_REASON_NONE;
+      TP_FILE_TRANSFER_STATE_CHANGE_REASON_NONE;
 
   tp_file->priv->transferred_bytes = g_value_get_uint64 (
       g_hash_table_lookup (properties, "TransferredBytes"));
@@ -621,7 +624,7 @@ tp_file_channel_set_dbus_property (gpointer proxy,
 {
         DEBUG ("Setting %s property", property);
         tp_cli_dbus_properties_call_set (TP_PROXY (proxy), -1,
-            EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, property, value,
+            TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, property, value,
             NULL, NULL, NULL, NULL);
 }
 
@@ -735,13 +738,14 @@ empathy_tp_file_get_channel (EmpathyTpFile *tp_file)
 }
 
 static void
-tp_file_method_cb (TpProxy *proxy,
+tp_file_method_cb (TpChannel *channel,
                    const GValue *address,
                    const GError *error,
                    gpointer user_data,
                    GObject *weak_object)
 {
   EmpathyTpFile *tp_file = (EmpathyTpFile *) weak_object;
+  GArray *array;
 
   if (error)
     {
@@ -750,11 +754,37 @@ tp_file_method_cb (TpProxy *proxy,
       return;
     }
 
-  tp_file->priv->unix_socket_path = g_value_dup_string (address);
+  if (G_VALUE_TYPE (address) == DBUS_TYPE_G_UCHAR_ARRAY)
+    {
+      tp_file->priv->socket_address = tp_g_value_slice_dup (address);
+    }
+  else if (G_VALUE_TYPE (address) == G_TYPE_STRING)
+    {
+      /* Old bugged version of telepathy-salut used to store the address
+       * as a 's' instead of an 'ay' */
+      const gchar *path;
 
-  DEBUG ("Got unix socket path: %s", tp_file->priv->unix_socket_path);
+      path = g_value_get_string (address);
+      array = g_array_sized_new (TRUE, FALSE, sizeof (gchar), strlen (path));
+      g_array_insert_vals (array, 0, path, strlen (path));
 
-  if (tp_file->priv->state == EMP_FILE_TRANSFER_STATE_OPEN)
+      tp_file->priv->socket_address = tp_g_value_slice_new (
+          DBUS_TYPE_G_UCHAR_ARRAY);
+      g_value_set_boxed (tp_file->priv->socket_address, array);
+
+      g_array_free (array, TRUE);
+    }
+  else
+    {
+      DEBUG ("Wrong address type: %s", G_VALUE_TYPE_NAME (address));
+      empathy_tp_file_cancel (tp_file);
+      return;
+    }
+
+  array = g_value_get_boxed (tp_file->priv->socket_address);
+  DEBUG ("Got unix socket path: %s", array->data);
+
+  if (tp_file->priv->state == TP_FILE_TRANSFER_STATE_OPEN)
     tp_file_start_transfer (tp_file);
 }
 
@@ -766,7 +796,7 @@ tp_file_method_cb (TpProxy *proxy,
  * @error: a #GError set if there is an error when opening @gfile
  *
  * Accepts a file transfer that's in the "local pending" state (i.e.
- * EMP_FILE_TRANSFER_STATE_LOCAL_PENDING).
+ * TP_FILE_TRANSFER_STATE_LOCAL_PENDING).
  */
 void
 empathy_tp_file_accept (EmpathyTpFile *tp_file,
@@ -792,8 +822,7 @@ empathy_tp_file_accept (EmpathyTpFile *tp_file,
   g_value_init (&nothing, G_TYPE_STRING);
   g_value_set_static_string (&nothing, "");
 
-  emp_cli_channel_type_file_transfer_call_accept_file (TP_PROXY (
-      tp_file->priv->channel),
+  tp_cli_channel_type_file_transfer_call_accept_file (tp_file->priv->channel,
       -1, TP_SOCKET_ADDRESS_TYPE_UNIX, TP_SOCKET_ACCESS_CONTROL_LOCALHOST,
       &nothing, offset, tp_file_method_cb, NULL, NULL, G_OBJECT (tp_file));
 }
@@ -805,7 +834,7 @@ empathy_tp_file_accept (EmpathyTpFile *tp_file,
  * @error: a #GError set if there is an error when opening @gfile
  *
  * Offers a file transfer that's in the "not offered" state (i.e.
- * EMP_FILE_TRANSFER_STATE_NOT_OFFERED).
+ * TP_FILE_TRANSFER_STATE_NOT_OFFERED).
  */
 void
 empathy_tp_file_offer (EmpathyTpFile *tp_file, GFile *gfile, GError **error)
@@ -821,9 +850,8 @@ empathy_tp_file_offer (EmpathyTpFile *tp_file, GFile *gfile, GError **error)
   g_value_init (&nothing, G_TYPE_STRING);
   g_value_set_static_string (&nothing, "");
 
-  emp_cli_channel_type_file_transfer_call_provide_file (
-      TP_PROXY (tp_file->priv->channel), -1,
-      TP_SOCKET_ADDRESS_TYPE_UNIX, TP_SOCKET_ACCESS_CONTROL_LOCALHOST,
+  tp_cli_channel_type_file_transfer_call_provide_file (tp_file->priv->channel,
+      -1, TP_SOCKET_ADDRESS_TYPE_UNIX, TP_SOCKET_ACCESS_CONTROL_LOCALHOST,
       &nothing, tp_file_method_cb, NULL, NULL, G_OBJECT (tp_file));
 }
 
@@ -848,12 +876,12 @@ empathy_tp_file_is_incoming (EmpathyTpFile *tp_file)
   return tp_file->priv->incoming;
 }
 
-EmpFileTransferState
+TpFileTransferState
 empathy_tp_file_get_state (EmpathyTpFile *tp_file,
-                           EmpFileTransferStateChangeReason *reason)
+                           TpFileTransferStateChangeReason *reason)
 {
   g_return_val_if_fail (EMPATHY_IS_TP_FILE (tp_file),
-      EMP_FILE_TRANSFER_STATE_NONE);
+      TP_FILE_TRANSFER_STATE_NONE);
 
   if (reason != NULL)
     *reason = tp_file->priv->state_change_reason;
@@ -913,12 +941,20 @@ empathy_tp_file_cancel (EmpathyTpFile *tp_file)
 {
   g_return_if_fail (EMPATHY_IS_TP_FILE (tp_file));
 
-  tp_cli_channel_call_close (tp_file->priv->channel, -1, NULL, NULL, NULL, NULL);
+  DEBUG ("Closing channel..");
+  tp_cli_channel_call_close (tp_file->priv->channel, -1,
+    NULL, NULL, NULL, NULL);
 
   if (tp_file->priv->cancellable != NULL)
     g_cancellable_cancel (tp_file->priv->cancellable);
 }
 
+void
+empathy_tp_file_close (EmpathyTpFile *tp_file)
+{
+  empathy_tp_file_cancel (tp_file);
+}
+
 static void
 empathy_tp_file_class_init (EmpathyTpFileClass *klass)
 {
