@@ -38,7 +38,7 @@
 #include "empathy-contact.h"
 
 #define EMPATHY_GET_PRIV(obj,type) ((type##Priv*) ((type*)obj)->priv)
-#define G_STR_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
+#define EMP_STR_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
 
 G_BEGIN_DECLS
 
@@ -68,7 +68,7 @@ xmlNodePtr   empathy_xml_node_find_child_prop_value (xmlNodePtr       node,
 guint        empathy_account_hash                   (gconstpointer    key);
 gboolean     empathy_account_equal                  (gconstpointer    a,
 						    gconstpointer    b);
-MissionControl *empathy_mission_control_new         (void);
+MissionControl *empathy_mission_control_dup_singleton (void);
 const gchar * empathy_presence_get_default_message  (McPresence       presence);
 const gchar * empathy_presence_to_str               (McPresence       presence);
 McPresence    empathy_presence_from_str             (const gchar     *str);
@@ -87,22 +87,7 @@ McAccount *  empathy_channel_get_account            (TpChannel       *channel);
 gboolean     empathy_proxy_equal                    (gconstpointer    a,
 						     gconstpointer    b);
 guint        empathy_proxy_hash                     (gconstpointer    key);
-
-typedef void (*empathy_connection_callback_for_request_channel) (TpConnection *connection,
-								 TpChannel *channel,
-								 const GError *error,
-								 gpointer user_data,
-								 GObject *weak_object);
-void empathy_connection_request_channel (TpConnection *proxy,
-					 gint timeout_ms,
-				    	 const gchar *channel_type,
-					 guint handle_type,
-					 const gchar *name,
-					 gboolean suppress_handler,
-					 empathy_connection_callback_for_request_channel callback,
-					 gpointer user_data,
-					 GDestroyNotify destroy,
-					 GObject *weak_object);
+gboolean     empathy_check_available_state          (void);
 
 G_END_DECLS
 
